@@ -13,8 +13,7 @@
 #'
 #' @export
 fable <- function(..., response, distribution){
-  tsbl <- tsibble(...)
-  as_fable(tsbl, !!enquo(response), !!enquo(distribution))
+  build_fable(tsibble(...), !!enquo(response), !!enquo(distribution))
 }
 
 #' Is the object a fable
@@ -41,87 +40,65 @@ as_fable <- function(x, ...){
 #' @rdname as-fable
 #' @export
 as_fable.tbl_ts <- function(x, response, distribution, ...){
-  quo_response <- enquo(response)
-  response <- possibly(eval_tidy, NULL)(quo_response)
-  
-  # If the response (from user input) needs converting
-  if(!is.list(response)){
-    if(quo_is_call(quo_response) && call_name(quo_response) == "c"){
-      response[[1]] <- rlang::exprs
-      response <- eval_tidy(quo_response)
-    }
-    else{
-      response <- list(get_expr(quo_response))
-    }
-  }
-  
-  fbl <- new_tsibble(x, class = "fbl_ts",
-                     response = response, dist = enexpr(distribution),
-                     model_cn = ".model")
-  validate_fable(fbl)
-  fbl
+  build_fable(x, 
+              response = !!enquo(response),
+              distribution = !!enquo(distribution))
 }
 
 #' @rdname as-fable
 #' @export
-as_fable.grouped_ts <- function(x, response, distribution, ...){
-  quo_response <- enquo(response)
-  response <- possibly(eval_tidy, NULL)(quo_response)
-  
-  # If the response (from user input) needs converting
-  if(!is.list(response)){
-    if(quo_is_call(quo_response) && call_name(quo_response) == "c"){
-      quo_response[[1]] <- rlang::exprs
-      response <- eval_tidy(quo_response)
-    }
-    else{
-      response <- list(get_expr(quo_response))
-    }
-  }
-  
-  fbl <- structure(x, class = c("grouped_fbl", "grouped_ts", "grouped_df", 
-                                "fbl_ts", "tbl_ts", "tbl_df", "tbl", "data.frame"),
-                   response = response, dist = enexpr(distribution),
-                   model_cn = ".model")
-  validate_fable(fbl)
-  fbl
-}
+as_fable.grouped_ts <- as_fable.tbl_ts
 
 #' @rdname as-fable
 #' @export
 as_fable.tbl_df <- function(x, response, distribution, ...){
-  as_fable(as_tsibble(x, ...), response = !!enquo(response),
-           distribution = !!enexpr(distribution))
+  build_fable(as_tsibble(x, ...), 
+              response = !!enquo(response),
+              distribution = !!enquo(distribution))
 }
 
 #' @rdname as-fable
 #' @export
 as_fable.fbl_ts <- function(x, response, distribution, ...){
   if(missing(response)){
-    response <- x%@%"response"
+    response <- response_vars(x)
   }
   else{
-    quo_response <- enquo(response)
-    # If the response (from user input) needs converting
-    if(quo_is_call(quo_response) && call_name(quo_response) == "c"){
-      response[[1]] <- rlang::exprs
-      response <- eval_tidy(quo_response)
-    }
-    else{
-      response <- list(get_expr(quo_response))
-    }
+    response <- eval_tidy(enquo(response))
   }
   if(missing(distribution)){
-    distribution <- x%@%"dist"
+    distribution <- distribution_var(x)
   }
-  as_fable(update_tsibble(x, ...), response = response,
-           distribution = !!enexpr(distribution))
+  else{
+    distribution <- names(x)[tidyselect::eval_select(enquo(distribution), x)]
+  }
+  build_fable(update_tsibble(x, ...),
+              response = response, distribution = distribution)
 }
 
 #' @rdname as-fable
 #' @export
 as_fable.grouped_df <- as_fable.tbl_df
 
+build_fable <- function (x, response, distribution) {
+  # If the response (from user input) needs converting
+  response <- eval_tidy(enquo(response))
+  distribution <- names(x)[tidyselect::eval_select(enquo(distribution), x)]
+  
+  if(is_grouped_ts(x)){
+    fbl <- structure(x, class = c("grouped_fbl", "grouped_ts", "grouped_df", 
+                                  "fbl_ts", "tbl_ts", "tbl_df", "tbl", "data.frame"),
+                     response = response, dist = distribution,
+                     model_cn = ".model")
+  } else {
+    fbl <- tsibble::new_tsibble(
+      x, response = response, dist = distribution, model_cn = ".model",
+      class = "fbl_ts")
+  }
+  validate_fable(fbl)
+  fbl
+}
+
 #' @export
 as_tsibble.fbl_ts <- function(x, ...){
   new_tsibble(x)
@@ -133,21 +110,24 @@ as_tsibble.grouped_fbl <- function(x, ...){
             response = NULL, dist = NULL, model_cn = NULL)
 }
 
+#' @export
+as_tibble.fbl_ts <- function(x, ...) {
+  new_tibble(vec_data(x), nrow = nrow(x))
+}
+
+#' @export
+as_tibble.grouped_fbl <- function(x, ...) {
+  dplyr::new_grouped_df(as_tibble(vec_data(x)), groups = group_data(x))
+}
+
 validate_fable <- function(fbl){
   stopifnot(inherits(fbl, "fbl_ts"))
-  chr_resp <- map_chr(attr(fbl, "response"), expr_text)
-  chr_dist <- as_string(attr(fbl, "dist"))
-  if (!all(chr_resp %in% names(fbl))){
-    bad_resp <- paste0(setdiff(chr_resp, names(fbl)), collapse = ", ")
-    abort(sprintf("Could not find response variable(s) in the fable: %s", bad_resp))
-  }
+  chr_dist <- distribution_var(fbl)
   if (!(chr_dist %in% names(fbl))){
     abort(sprintf("Could not find distribution variable `%s` in the fable. A fable must contain a distribution, if you want to remove it convert to a tsibble with `as_tsibble()`.",
                   chr_dist))
   }
-  if (!inherits(fbl[[chr_dist]], "fcdist")){
-    abort('Distribution variable must be of class "fcdist"')
-  }
+  vec_is(fbl[[chr_dist]], distributional::new_dist())
 }
 
 tbl_sum.fbl_ts <- function(x){
@@ -156,34 +136,60 @@ tbl_sum.fbl_ts <- function(x){
   out
 }
 
-#' @rdname hilo
+#' @importFrom distributional hilo
 #' @export
 hilo.fbl_ts <- function(x, level = c(80, 95), ...){
-  x %>%
-    transmute(
-      !!!(x%@%"response"),
-      !!!set_names(map(level,function(.x) expr(hilo(!!(x%@%"dist"), !!.x))),
+  as_tsibble(x) %>%
+    mutate(
+      !!!set_names(map(level,function(.x) expr(hilo(!!sym(distribution_var(x)), !!.x))),
                    paste0(level, "%"))
     )
 }
 
+restore_fable <- function(data, template){
+  data <- as_tibble(data)
+  data_cols <- names(data)
+  
+  # key_vars <- setdiff(key_vars(template), data_cols)
+  # key_data <- select(key_data(template), key_vars)
+  # if (vec_size(key_data) == 1) {
+  #   template <- remove_key(template, setdiff(key_vars(template), key_vars))
+  # }
+  
+  # Variables to keep
+  tsbl_vars <- setdiff(c(index_var(template), key_vars(template)), data_cols)
+  fbl_vars <- setdiff(distribution_var(template), data_cols)
+  res <- bind_cols(template[tsbl_vars], data, template[fbl_vars])
+  
+  tsbl <- build_tsibble(res, !!key_vars(template), 
+                        index = !!index(template), index2 = !!index2(template),
+                        ordered = is_ordered(template), interval = interval(template),
+                        validate = FALSE)
+  
+  build_fable(tsbl, response = response_vars(template), distribution = !!distribution_var(template))
+}
+
 #' @export
 select.fbl_ts <- function (.data, ...){
-  as_fable(NextMethod(), .data%@%"response", !!(.data%@%"dist"))
+  res <- select(as_tibble(.data), ...)
+  restore_fable(res, .data)
 }
 
 #' @export
 select.grouped_fbl <- select.fbl_ts
 
-filter.fbl_ts <- function (.data, ...){
-  as_fable(NextMethod(), .data%@%"response", !!(.data%@%"dist"))
+#' @export
+transmute.fbl_ts <- function (.data, ...) {
+  res <- transmute(as_tsibble(.data), ...)
+  restore_fable(res, .data)
 }
 
-filter.grouped_fbl <- filter.fbl_ts
+#' @export
+transmute.grouped_fbl <- transmute.fbl_ts
 
 #' @export
 group_by.fbl_ts <- function(.data, ...) {
-  as_fable(NextMethod(), .data%@%"response", !!(.data%@%"dist"))
+  build_fable(NextMethod(), response_vars(.data), distribution_var(.data))
 }
 
 #' @export
@@ -195,33 +201,34 @@ ungroup.fbl_ts <- group_by.fbl_ts
 #' @export
 ungroup.grouped_fbl <- group_by.fbl_ts
 
-#' @export
-mutate.fbl_ts <- function(.data, ...) {
-  as_fable(NextMethod(), .data%@%"response", !!(.data%@%"dist"))
-}
-
-#' @export
-mutate.grouped_fbl <- mutate.fbl_ts
-
 #' @export
 rbind.fbl_ts <- function(...){
+  .Deprecated("bind_rows()")
   fbls <- dots_list(...)
-  response <- map(fbls, attr, "response")
-  dist <- map(fbls, attr, "dist")
+  response <- map(fbls, response_vars)
+  dist <- map(fbls, distribution_var)
   if(length(response <- unique(response)) > 1){
     abort("Cannot combine fables with different response variables.")
   }
   if(length(dist <- unique(dist)) > 1){
     abort("Cannot combine fables with different distribution names.")
   }
-  out <- suppressWarnings(invoke("rbind", map(fbls, as_tsibble)))
-  class(out[[as_string(dist[[1]])]]) <- c("fcdist", "list")
-  as_fable(out, response[[1]], !!dist[[1]])
+  out <- suppressWarnings(invoke(bind_rows, map(fbls, as_tsibble)))
+  build_fable(out, response[[1]], dist[[1]])
 }
 
 #' @export
 `[.fbl_ts` <- function (x, i, j, drop = FALSE){
-  as_fable(NextMethod(), x%@%"response", !!(x%@%"dist"))
+  out <- NextMethod()
+  # Drop fable if tsibble is dropped
+  
+  cn <- colnames(out)
+  not_fable <- !(distribution_var(x) %in% cn) || !is_tsibble(out)
+  
+  if(not_fable)
+    return(out)
+  else
+    build_fable(out, response_vars(x), distribution_var(x))
 }
 
 type_sum.fbl_ts <- function(x){
