@@ -29,15 +29,15 @@
 #' @export
 generate.mdl_df <- function(x, new_data = NULL, h = NULL, times = 1, seed = NULL, ...){
   kv <- c(key_vars(x), ".model")
-  mdls <- x%@%"models"
+  mdls <- mable_vars(x)
   if(!is.null(new_data)){
-    x <- bind_new_data(x, new_data)
+    new_data <- bind_new_data(x, new_data)[["new_data"]]
   }
   x <- as_tibble(gather(x, ".model", ".sim", !!!syms(mdls)))
   
   # Evaluate simulations
   x$.sim <- map2(x[[".sim"]], 
-                 x[["new_data"]] %||% rep(list(NULL), length.out = NROW(x)),
+                 new_data %||% rep(list(NULL), length.out = NROW(x)),
                  generate, h = h, times = times, seed = seed, ...)
   unnest_tsbl(x, ".sim", parent_key = kv)
 }
@@ -61,12 +61,14 @@ generate.mdl_ts <- function(x, new_data = NULL, h = NULL, times = 1, seed = NULL
   }
   
   if(is.null(new_data[[".rep"]])){
-    new_data <- map(seq_len(times), function(rep){
-      new_data[[".rep"]] <- rep
-      update_tsibble(new_data, key = c(".rep", key_vars(new_data)),
-                     validate = FALSE)
-    }) %>% 
-      invoke("rbind", .)
+    kv <- c(".rep", key_vars(new_data))
+    idx <- index_var(new_data)
+    intvl <- tsibble::interval(new_data)
+    new_data <- vctrs::vec_rbind(
+      !!!set_names(rep(list(as_tibble(new_data)), times), seq_len(times)),
+      .names_to = ".rep"
+    )
+    new_data <- build_tsibble(new_data, index = !!idx, key = !!kv, interval = intvl)
   }
   
   # Compute specials with new_data
